<?php
/*
 * 公共函数库
 *
 */

/**
 * 随机数生成
 * @param string $type 类型 alpha/alnum/numeric/nozero/unique/md5/encrypt/sha1
 * @param int $len 长度
 * @return string
 */
function getKey($type = 'alnum', $len = 8)
{
    switch ($type) {
        case 'alpha':
        case 'alnum':
        case 'numeric':
        case 'nozero':
            switch ($type) {
                case 'alpha':
                    $pool = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
                    break;
                case 'alnum':
                    $pool = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
                    break;
                case 'numeric':
                    $pool = '0123456789';
                    break;
                case 'nozero':
                    $pool = '123456789';
                    break;
            }
            return substr(str_shuffle(str_repeat($pool, ceil($len / strlen($pool)))), 0, $len);
        case 'unique':
        case 'md5':
            return md5(uniqid(mt_rand()));
        case 'encrypt':
        case 'sha1':
            return sha1(uniqid(mt_rand(), true));
    }
}


/**
 * CURL发送Request请求,含POST和REQUEST
 * @param string $url 请求的链接
 * @param mixed $params 传递的参数
 * @param string $method 请求的方法
 * @param mixed $options CURL的参数
 * @return array
 */
function sendRequest($url, $params = array(), $method = 'POST', $options = array())
{
    $method = strtoupper($method);
    $protocol = substr($url, 0, 5);
    $query_string = is_array($params) ? http_build_query($params) : $params;

    $ch = curl_init();
    $defaults = array();
    if ('GET' == $method) {
        $geturl = $query_string ? $url . (stripos($url, "?") !== false ? "&" : "?") . $query_string : $url;
        $defaults[CURLOPT_URL] = $geturl;
    } else {
        $defaults[CURLOPT_URL] = $url;
        if ($method == 'POST') {
            $defaults[CURLOPT_POST] = 1;
        } else {
            $defaults[CURLOPT_CUSTOMREQUEST] = $method;
        }
        $defaults[CURLOPT_POSTFIELDS] = $query_string;
    }

    $defaults[CURLOPT_HEADER] = false;
    $defaults[CURLOPT_USERAGENT] = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/45.0.2454.98 Safari/537.36";
    $defaults[CURLOPT_FOLLOWLOCATION] = true;
    $defaults[CURLOPT_RETURNTRANSFER] = true;
    $defaults[CURLOPT_CONNECTTIMEOUT] = 3;
    $defaults[CURLOPT_TIMEOUT] = 3;

    // disable 100-continue
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));

    if ('https' == $protocol) {
        $defaults[CURLOPT_SSL_VERIFYPEER] = false;
        $defaults[CURLOPT_SSL_VERIFYHOST] = false;
    }

    curl_setopt_array($ch, (array)$options + $defaults);

    $ret = curl_exec($ch);
    $err = curl_error($ch);

    if (false === $ret || !empty($err)) {
        $errno = curl_errno($ch);
        $info = curl_getinfo($ch);
        curl_close($ch);
        return array(
            'ret' => false,
            'errno' => $errno,
            'msg' => $err,
            'info' => $info,
        );
    }
    curl_close($ch);
    return array(
        'ret' => true,
        'msg' => $ret,
    );
}
/**
 * 发送一个POST请求
 */
function sendPost($url, $params = array(), $options =array())
{
    $req = sendRequest($url, $params, 'POST', $options);
    return $req['ret'] ? $req['msg'] : '';
}
/**
 * 发送一个GET请求
 */
function sendGet($url, $params = array(), $options = array())
{
    $req = sendRequest($url, $params, 'GET', $options);
    return $req['ret'] ? $req['msg'] : '';
}
/**
 * 返回json数据
 *
 * @param [type] $code
 * @param [type] $msg
 * @param array $data
 * @return void
 */
function json($code, $msg, $data=array())
{
    return array(
        'code'=>$code,
        'msg'=>$msg,
        'data'=>$data
    );
}

function getIndex()
{
    return 'http://' . $_SERVER['SERVER_NAME'];
}
/**
 * 获取全局挂载按钮的位置
 *
 * @param int $type
 * @return array
 */
function getButtonAddress($type)
{
    switch ($type) {
        case 1:
            return array('left','top');
            break;
        case 2:
            return array('right','top');
            break;
        case 3:
            return array('left','bottom');
            break;
        case 4:
            return array('right','bottom');
            break;
    }
}
/**
 * 下载文件
 *
 * @param string $url
 * @param string $fileName
 * @param string $path
 * @return bool
 */
function downFileToPath($url, $fileName)
{
    //下载时候不中断等待
    @set_time_limit(0);
    //DISCUZ_ROOT.'/source/plugin/yimen_app/updata/'.
    if ($fp = @fopen($url, "rb")) {
        if (!$downloadFile = @fopen($fileName, "wb")) {
            return false;
        }
        while (!feof($fp)) {
            if (!file_exists($fileName)) {
                fclose($downloadFile);
                return false;
            }
            clearstatcache();
            fwrite($downloadFile, fread($fp, 1024 * 200), 1024 * 200);
        }
        fclose($downloadFile);
        fclose($fp);
        return true;
    } else {
        return false;
    }
}
function getSecond($url, $new_file)
{
    $header = array(
        'User-Agent: Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:45.0) Gecko/20100101 Firefox/45.0',
        'Accept-Language: zh-CN,zh;q=0.8,en-US;q=0.5,en;q=0.3',
        'Accept-Encoding: gzip, deflate',);
    $curl = curl_init();

    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
    curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
    $data = curl_exec($curl);
    $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);

    curl_close($curl);
    if ($code == 200) {//把URL格式的图片转成base64_encode格式的！
        $imgBase64Code = "data:image/jpeg;base64," . base64_encode($data);
    }
    $img_content=$imgBase64Code;//图片内容
    //echo $img_content;exit;
    if (preg_match('/^(data:\s*image\/(\w+);base64,)/', $img_content, $result)) {
        $type = $result[2];//得到图片类型png?jpg?gif?
        if (file_put_contents($new_file, base64_decode(str_replace($result[1], '', $img_content)))) {
            return $new_file;
        }
    }
}
/**
 * 获取当前插件版本号
 *
 * @return int
 */
function getPluginVersion()
{
    return 20190926114509;
}
/**
 * 获取程序编码
 *
 * @return string
 */
function getChar()
{
    global $_G;
    $arr=array('utf-8'=>'utf8','gbk'=>'gbk');
    return $arr[$_G['charset']];
}
function setChar($char)
{
    global $_G;
    if ($_G['charset']=='utf-8') {
        return $char;
    } else {
        return iconv('gbk', 'utf-8', $char);
    }
}

/**
 * 获取微信授权用户数据
 *
 * @param string $appid
 * @param string $appsecret
 * @param string $code
 * @return array
 */
function getWechatUserInfo($appid, $appsecret, $code)
{
    $tokenInfo=sendGet(
        'https://api.weixin.qq.com/sns/oauth2/access_token',
        array('appid'=>$appid,'secret'=>$appsecret,'code'=>$code,'grant_type'=>'authorization_code')
    );
    $tokenInfo=json_decode($tokenInfo, true);
    if (empty($tokenInfo['access_token'])) {
        return false;
    }
    $accessToken=$tokenInfo['access_token'];
    $openid=$tokenInfo['openid'];
    $userInfo=sendGet(
        'https://api.weixin.qq.com/sns/userinfo',
        array('access_token'=>$accessToken,'openid'=>$openid)
    );
    $userInfo=json_decode($userInfo, true);
    if (empty($userInfo['openid'])) {
        return false;
    }
    $unionid='';
    if (!empty($userInfo['unionid'])) {
        $unionid=$userInfo['unionid'];
    }
    return array(
        'openid'=>$openid,
        'unionid'=>$unionid,
        'nick'=>$userInfo['nickname'],
        'avatar'=>$userInfo['headimgurl']
    );
}

/**
 * 获取QQ Web登陆 用户数据
 *
 * @param string $appid
 * @param string $appsecret
 * @param string $code
 * @param string $callback
 * @return array
 */
function getWebQQUserInfo($appid, $appsecret, $code, $callback)
{
    $tokenInfo1=sendGet(
        'https://graph.qq.com/oauth2.0/token?grant_type=authorization_code&client_id='.$appid.'&client_secret='.$appsecret.'&code='.$code.'&redirect_uri='. urlencode($callback)
    );
    parse_str($tokenInfo1, $tokenInfo);
    if (empty($tokenInfo['access_token'])) {
        return false;
    }
    $accessToken=$tokenInfo['access_token'];
    $url='';
    $openInfo=sendGet('https://graph.qq.com/oauth2.0/me?access_token='.$accessToken);
    $l = strpos($openInfo, "(");
    $r = strrpos($openInfo, ")");
    $openInfo = substr($openInfo, $l + 1, $r - $l -1);
    $openInfo=json_decode($openInfo, true);
    if (empty($openInfo['openid'])) {
        return false;
    }
    //获取unionid
    $uInfo=sendGet('https://graph.qq.com/oauth2.0/me?access_token='.$accessToken.'&unionid=1');
    $l = strpos($uInfo, "(");
    $r = strrpos($uInfo, ")");
    $uInfo = substr($uInfo, $l + 1, $r - $l -1);
    $uInfo=json_decode($uInfo, true);
    $openid=$openInfo['openid'];
    $unionid='';
    if (!empty($uInfo['unionid'])) {
        $unionid=$uInfo['unionid'];
    }
    //获取用户信息
    $userInfo=sendGet(
        'https://graph.qq.com/user/get_user_info',
        array('access_token'=>$accessToken,'openid'=>$openid,'oauth_consumer_key'=>$appid)
    );
    $userInfo=json_decode($userInfo, true);
    if (empty($userInfo['nickname'])) {
        return false;
    }
    $nick=$userInfo['nickname'];
    $avatar=$userInfo['figureurl_qq_2'];
    return array(
        'openid'=>$openid,
        'unionid'=>$unionid,
        'nick'=>$nick,
        'avatar'=>$avatar
    );
}
//微信支付
function getWechatPay($setting, $amount, $order_id)
{
    //获取微信支付的QR
    $wx_appid=$setting['wechat_pay']['wx_appid'];      //微信开放平台appid
    $wx_partnerid=$setting['wechat_pay']['wx_partnerid'];   //微信商户平台商户id
    $wx_key=$setting['wechat_pay']['wx_key'];
    require "core/wechat/lib/WxPay.Config.php";
    require "core/wechat/lib/WxPay.Api.php";
    require "core/wechat/example/WxPay.NativePay.php";
    $notify = new NativePay();
    $input = new WxPayUnifiedOrder();
    $input->SetBody(setChar('订单号:'.$order_id));
    $input->SetAttach("app_pay");
    $input->SetOut_trade_no($order_id);
    $input->SetTotal_fee($amount*100);
    $input->SetTime_start(date("YmdHis"));
    $input->SetTime_expire(date("YmdHis", time() + 600));
    $input->SetNotify_url('http://paysdk.weixin.qq.com/example/notify.php');
    $input->SetTrade_type("NATIVE");
    $input->SetProduct_id($amount*1000);
    $result = $notify->GetPayUrl($input);
    return $result;
}
function getWechatStutus($setting, $order_id)
{
    $wx_appid=$setting['wechat_pay']['wx_appid'];      //微信开放平台appid
    $wx_partnerid=$setting['wechat_pay']['wx_partnerid'];   //微信商户平台商户id
    $wx_key=$setting['wechat_pay']['wx_key'];
    require "core/wechat/lib/WxPay.Config.php";
    require "core/wechat/lib/WxPay.Api.php";
    $input = new WxPayOrderQuery();
    $input->SetOut_trade_no($order_id);
    return WxPayApi::orderQuery($input);
}
function is_mobile_request()
{
    $_SERVER['ALL_HTTP'] = isset($_SERVER['ALL_HTTP']) ? $_SERVER['ALL_HTTP'] : '';
    $mobile_browser = '0';
    if (preg_match('/(up.browser|up.link|mmp|symbian|smartphone|midp|wap|phone|iphone|ipad|ipod|android|xoom)/i', strtolower($_SERVER['HTTP_USER_AGENT']))) {
        $mobile_browser++;
    }
    if ((isset($_SERVER['HTTP_ACCEPT'])) and (strpos(strtolower($_SERVER['HTTP_ACCEPT']), 'application/vnd.wap.xhtml+xml') !== false)) {
        $mobile_browser++;
    }
    if (isset($_SERVER['HTTP_X_WAP_PROFILE'])) {
        $mobile_browser++;
    }
    if (isset($_SERVER['HTTP_PROFILE'])) {
        $mobile_browser++;
    }
    $mobile_ua = strtolower(substr($_SERVER['HTTP_USER_AGENT'], 0, 4));
    $mobile_agents = array(
        'w3c ','acs-','alav','alca','amoi','audi','avan','benq','bird','blac',
        'blaz','brew','cell','cldc','cmd-','dang','doco','eric','hipt','inno',
        'ipaq','java','jigs','kddi','keji','leno','lg-c','lg-d','lg-g','lge-',
        'maui','maxo','midp','mits','mmef','mobi','mot-','moto','mwbp','nec-',
        'newt','noki','oper','palm','pana','pant','phil','play','port','prox',
        'qwap','sage','sams','sany','sch-','sec-','send','seri','sgh-','shar',
        'sie-','siem','smal','smar','sony','sph-','symb','t-mo','teli','tim-',
        'tosh','tsm-','upg1','upsi','vk-v','voda','wap-','wapa','wapi','wapp',
        'wapr','webc','winw','winw','xda','xda-'
        );
    if (in_array($mobile_ua, $mobile_agents)) {
        $mobile_browser++;
    }
    if (strpos(strtolower($_SERVER['ALL_HTTP']), 'operamini') !== false) {
        $mobile_browser++;
    }
    // Pre-final check to reset everything if the user is on Windows
    if (strpos(strtolower($_SERVER['HTTP_USER_AGENT']), 'windows') !== false) {
        $mobile_browser=0;
    }
    // But WP7 is also Windows, with a slightly different characteristic
    if (strpos(strtolower($_SERVER['HTTP_USER_AGENT']), 'windows phone') !== false) {
        $mobile_browser++;
    }
    if ($mobile_browser>0) {
        return true;
    } else {
        return false;
    }
}
